import { pub } from "../../../toolbox/pub.js";
import { onesie } from "../../../toolbox/onesie.js";
import { ops } from "../../../framework/ops.js";
import { makeChatState } from "./state/chat-state.js";
import { chatAllowance } from "../common/chat-allowance.js";
import { setupRoomManagement } from "./room/room-management.js";
import { makeStatsFetcher } from "./utilities/make-stats-fetcher.js";
import { makeChatClientside } from "../api/services/chat-clientside.js";
export function makeChatModel({ chatConnect, getChatMeta }) {
    const changeEvent = pub();
    const state = makeChatState();
    const clientsideApi = makeChatClientside({
        state,
        onChange: changeEvent.publish,
    });
    const statsFetcher = makeStatsFetcher({ state, intervalDuration: 2000 });
    const reconnect = onesie(async function () {
        var _a;
        const connection = (_a = ops.value(state.readable.connectionOp)) !== null && _a !== void 0 ? _a : await ops.operation({
            setOp: op => state.writable.connectionOp = op,
            promise: chatConnect({
                clientsideApi,
                handleDisconnect: () => state.writable.connectionOp = ops.none(),
            }),
        });
        const meta = await getChatMeta();
        await connection.serverside.chatServer.updateUserMeta(meta);
        statsFetcher.startInterval(connection);
        return connection;
    });
    async function disconnect() {
        statsFetcher.stopInterval();
        const connection = ops.value(state.readable.connectionOp);
        if (connection) {
            connection.disconnect();
            state.writable.connectionOp = ops.none();
        }
    }
    const roomManagement = setupRoomManagement({
        state,
        reconnect,
        disconnect,
    });
    return {
        state: state.readable,
        subscribe: state.subscribe,
        subscribeToChange: changeEvent.subscribe,
        get allowance() {
            var _a;
            const access = ops.value(state.readable.accessOp);
            const privileges = (_a = access === null || access === void 0 ? void 0 : access.permit.privileges) !== null && _a !== void 0 ? _a : [];
            return chatAllowance(privileges);
        },
        async updateAccessOp(op) {
            state.writable.accessOp = op;
            const access = ops.value(op);
            await roomManagement.updateAuthSituation(!!access);
            const connection = ops.value(state.readable.connectionOp);
            if (connection) {
                const meta = await getChatMeta();
                await connection.serverside.chatServer.updateUserMeta(meta);
            }
        },
        session: roomManagement.getRoomSession,
        disconnect,
        reconnect,
    };
}
//# sourceMappingURL=chat-model.js.map