import * as dbmage from "dbmage";
import { getStripeId } from "../stripe/utils/get-stripe-id.js";
export async function deleteTiersAndRolesForMissingStripeProducts({ auth, tierRows, stripeProducts }) {
    const { storeDatabase, roleManager } = auth;
    const storeTables = storeDatabase.tables;
    const stripeProductIds = stripeProducts.map(product => getStripeId(product.id));
    const idsForMissingTiers = tierRows
        .filter(row => !stripeProductIds.includes(row.stripeProductId))
        .map(row => ({ roleId: row.roleId, tierId: row.tierId }));
    if (idsForMissingTiers.length) {
        await storeTables
            .subscriptions
            .tiers
            .delete(dbmage.findAll(idsForMissingTiers, ({ tierId }) => ({ tierId })));
        await Promise.all(idsForMissingTiers.map(async ({ roleId }) => {
            await roleManager.deleteRoleAndAllRelatedRecords(roleId);
        }));
    }
}
//# sourceMappingURL=delete-tiers-and-roles-for-missing-stripe-products.js.map