import { getStripeId } from "../../stripe/utils/get-stripe-id.js";
import { buildFinalTierObject } from "./build-final-tier-object.js";
import { isTierBelongingToPlan } from "./is-tier-belonging-to-plan.js";
import { isTierAttachedToExistingStripeProduct } from "./is-tier-attached-to-existing-stripe-product.js";
export function compileTiers(tierRows, planRow, stripeProducts, stripePrices) {
    const stripeProductIds = (stripeProducts
        .map(product => getStripeId(product.id)));
    function findPriceForTier(tierRow) {
        return stripePrices
            .find(price => getStripeId(price === null || price === void 0 ? void 0 : price.product) === tierRow.stripeProductId);
    }
    function findProductForTier(tierRow) {
        return stripeProducts
            .find(product => getStripeId(product.id) === tierRow.stripeProductId);
    }
    return tierRows
        .filter(tierRow => isTierBelongingToPlan(planRow, tierRow))
        .filter(tierRow => isTierAttachedToExistingStripeProduct(tierRow, stripeProductIds))
        .map(tierRow => buildFinalTierObject(tierRow, findPriceForTier(tierRow), findProductForTier(tierRow)));
}
//# sourceMappingURL=compile-tiers.js.map